import argparse
import pymupdf

import tkinter
from tkinter import ttk
from PIL import Image, ImageTk, ImageDraw

CURRENT_PAGE = 0
RECT_START = None


class AddLinkDialog:
    def __init__(self, root, rect, toc) -> None:
        self.top = tkinter.Toplevel(root)
        frm = ttk.Frame(self.top, padding=10)
        frm.grid()

        toc_values = list(t for _, t, _ in toc)

        ttk.Label(frm, text=f"Rectangle: {rect!r}").grid(column=0, row=0, columnspan=3)
        ttk.Label(frm, text="Target:").grid(column=0, row=1)
        entry = ttk.Entry(frm)
        entry.grid(column=1, row=1, columnspan=2, sticky="EW")
        ttk.Button(frm, text="Add", command=self.top.destroy).grid(column=2, row=2)
        ttk.Button(frm, text="Cancel", command=self.top.destroy).grid(column=3, row=2)


def main() -> None:
    parser = argparse.ArgumentParser()
    parser.add_argument("file")
    args = parser.parse_args()

    doc: pymupdf.Document
    with pymupdf.open(args.file) as doc:
        toc = doc.get_toc()

        def toc_entry_for_page(page_num: int) -> str:
            last_title = ""
            for lvl, t, p in toc:
                if (p - 1) > page_num:
                    return last_title
                if (p - 1) == page_num:
                    return t
                last_title = t

        root = tkinter.Tk()
        root.title("PDF Navigation Annotator Tool")

        root.columnconfigure(0, weight=10)
        root.rowconfigure(0, weight=10)

        frm = ttk.Frame(root, padding=10)
        frm.grid(sticky="NSEW")

        frm.columnconfigure(2, weight=10)
        frm.rowconfigure(1, weight=10)

        columns = ("title", "page")
        toc_tree = ttk.Treeview(frm, columns=columns, show="headings")
        toc_tree.heading("title", text="Title")
        toc_tree.heading("page", text="Page")
        toc_tree.grid(column=0, row=0, rowspan=2, sticky="NS", padx=5, pady=5)

        for lvl, t, p in toc:
            toc_tree.insert("", tkinter.END, values=("  " * lvl + t, str(p)))

        page_title = ttk.Label(frm)
        page_title.grid(column=2, row=0)

        page_view = ttk.Label(frm)
        page_view.grid(column=2, row=1)

        def load_page(page_num: int, rect_overlay=None):
            page = doc[page_num]
            pix = page.get_pixmap(dpi=120)

            mode = "RGBA" if pix.alpha else "RGB"
            img = Image.frombytes(mode, [pix.width, pix.height], pix.samples)

            if rect_overlay is not None:
                draw = ImageDraw.Draw(img)
                draw.rectangle(rect_overlay, outline="red", width=2)

            img = ImageTk.PhotoImage(img)
            page_view.configure(image=img)
            page_view.image = img

            page_title.configure(text=toc_entry_for_page(page_num))

        def start_rect(event):
            global RECT_START
            if event.num != 1:
                return
            print(f"DOWN @ {event.x}/{event.y}")
            RECT_START = (event.x, event.y)

        def end_rect(event):
            global RECT_START
            if event.num != 1:
                return
            if RECT_START is not None:
                print(f"UP @ {event.x}/{event.y}")
                load_page(CURRENT_PAGE, [*RECT_START, event.x, event.y])
                AddLinkDialog(root, [*RECT_START, event.x, event.y], toc)
                RECT_START = None

        page_view.bind("<ButtonPress>", start_rect)
        page_view.bind("<ButtonRelease>", end_rect)


        def next_page():
            global CURRENT_PAGE
            CURRENT_PAGE += 1
            load_page(CURRENT_PAGE)

        def previous_page():
            global CURRENT_PAGE
            CURRENT_PAGE -= 1
            load_page(CURRENT_PAGE)

        ttk.Button(frm, text="Previous", command=previous_page).grid(
            column=1, row=1, sticky="NS", padx=5, pady=5
        )
        ttk.Button(frm, text="Next", command=next_page).grid(
            column=3, row=1, sticky="NS", padx=5, pady=5
        )

        root.bind("<Up>", lambda _: previous_page())
        root.bind("<Left>", lambda _: previous_page())
        root.bind("<Prior>", lambda _: previous_page())

        root.bind("<Down>", lambda _: next_page())
        root.bind("<Right>", lambda _: next_page())
        root.bind("<Next>", lambda _: next_page())

        root.bind("<Control-w>", lambda _: root.destroy())

        load_page(CURRENT_PAGE)

        root.mainloop()


if __name__ == "__main__":
    main()
